import numpy
import pylab

h = numpy.array([5.0,
                 3.0
                 ])

ComputationalTime = numpy.array([???,
                                 ???
                                 ])

nbnodes= numpy.array([???,
                ???
                ])

nbelem = numpy.array([???,
                      ???
                      ])

error = numpy.array([?????,
                     ?????
                     ])

MaxDisp=numpy.array([????,
                     ????
	             ])

VMmaxi=numpy.array([???,
                    ???
                    ])

VMmaxiSmooth=numpy.array([???,
                          ???
                          ])

pylab.figure(1)
pylab.plot(nbnodes,numpy.log10(MaxDisp))
pylab.scatter(nbnodes,numpy.log10(MaxDisp))
pylab.xlabel('Nb nodes')
pylab.ylabel('Disp. Maxi')
pylab.grid('on')

pylab.figure(2)
pylab.plot(nbnodes,VMmaxi,label='VM')
pylab.plot(nbnodes,VMmaxiSmooth,label='VM Smooth')
pylab.xlabel('Nb nodes')
pylab.ylabel('V.M. Maxi')
pylab.legend()
pylab.scatter(nbnodes,VMmaxi)
pylab.scatter(nbnodes,VMmaxiSmooth)
pylab.grid('on')

pylab.figure(3)
pylab.plot(numpy.log10(1.0/nbnodes),numpy.log10(error))
pylab.scatter(numpy.log10(1.0/nbnodes),numpy.log10(error))
pylab.xlabel('Average elemental length (log10)')
pylab.ylabel('Global error (log10)')
pylab.title('Global error convergence')
pylab.grid('on')

pylab.figure(4)
pylab.plot(numpy.log10(nbnodes),numpy.log10(ComputationalTime))
pylab.scatter(numpy.log10(nbnodes),numpy.log10(ComputationalTime))
pylab.xlabel('Nb nodes (log10)')
pylab.ylabel('Computational time (log10)')
pylab.grid('on')

pylab.show()




from SILEXlight import silex_lib_tet4_fortran as sx_tet4
import numpy as np


def test_compute_stress_strain_error():
    # sx_tet4.compute_stress_strain_error()
    assert True


def test_crossproduct():
    vA = np.random.randn(3)
    vB = np.random.randn(3)
    np.testing.assert_almost_equal(
        sx_tet4.crossproduct(vA, vB), np.cross(vA, vB))


def test_det33():
    mat = np.random.randn(3, 3)
    np.testing.assert_almost_equal(sx_tet4.det33(mat), np.linalg.det(mat))


def test_det33_ligne_de_un():
    mat = np.random.randn(3, 3)
    mat[0, :] = 1
    np.testing.assert_almost_equal(
        sx_tet4.det33_ligne_de_un(mat[1:3, :]), np.linalg.det(mat))


def test_det44_ligne_de_un():
    mat = np.random.randn(4, 4)
    mat[0, :] = 1
    np.testing.assert_almost_equal(
        sx_tet4.det44_ligne_de_un(mat[1:4, :]), np.linalg.det(mat))


def test_forceonsurface():
    # sx_tet4.forceonsurface()
    assert True


def test_normvector():
    vA = np.random.randn(3, 1)
    np.testing.assert_almost_equal(sx_tet4.normvector(vA,), np.linalg.norm(vA))


def test_stiffnessmatrix():
    # sx_tet4.stiffnessmatrix()
    assert True
    
def test_massmatrix():
    # sx_tet4.massmatrix()
    assert True



#!/usr/bin/env python
import os
def get_version(rel_path):
    """ Get version from version.py."""
    v = None
    vOk = False
    with open(os.path.join('./',rel_path),'r') as f:
        for line in f:
            if line.lstrip().startswith('__version__'):
                v = line.split('=')[-1].strip().replace("'", "").replace('"', "")
                vOk = True
                break
    if not vOk:
        raise RuntimeError("Unable to find version string.")
    return v

if __name__ == '__main__':
    print(get_version('SILEXlight/__init__.py'))


from SILEXlight import silex_lib_tet4_python as sx_tet4
import numpy as np


def test_compute_stress_strain_error():
    # sx_tet4.compute_stress_strain_error()
    assert True


def test_det33_ligne_de_un():
    mat = np.random.randn(3, 3)
    mat[0, :] = 1
    np.testing.assert_almost_equal(
        sx_tet4.det33_ligne_de_un(mat[1:3, :]), np.linalg.det(mat))


def test_det44_ligne_de_un():
    mat = np.random.randn(4, 4)
    mat[0, :] = 1
    np.testing.assert_almost_equal(
        sx_tet4.det44_ligne_de_un(mat[1:4, :]), np.linalg.det(mat))


def test_forceonsurface():
    # sx_tet4.forceonsurface()
    assert True


def test_stiffnessmatrix():
    # sx_tet4.stiffnessmatrix()
    assert True

def test_massmatrix():
    # sx_tet4.massmatrix()
    assert True


#############################################################################
#      Import libraries
#############################################################################
import string; import time; import numpy
import scipy.sparse
import scipy.sparse.linalg


## Choose between the Fortran or the Python librairie:

#import silex_lib_tri3 as silex_lib_elt
from SILEXlight import silex_lib_tri3_python as silex_lib_elt
from SILEXlight import silex_lib_gmsh

#install mumps to solve the system more quickly
#import mumps

#############################################################################
print("SILEX CODE - compute a plate with a hole with tri3 elements")
#############################################################################

#############################################################################
#      USER PART: Import mesh, boundary conditions and material
#############################################################################
tic = time.perf_counter()

# Input mesh: define the name of the mesh file (*.msh)
MeshFileName='plate-hole-tri3'

# Output result file: define the name of the result file (*.msh)
ResultsFileName='Results_plate-hole-tri3'

# choose the element type
eltype=2

# choose geometry dimension
ndim=2

# choose the results in the results file
flag_write_fields=1

# read the mesh from gmsh
nodes=silex_lib_gmsh.ReadGmshNodes(MeshFileName+'.msh',ndim)
elements,Idnodes=silex_lib_gmsh.ReadGmshElements(MeshFileName+'.msh',eltype,4)

# read surfaces where to impose boundary conditions
elementsS1,IdnodeS1=silex_lib_gmsh.ReadGmshElements(MeshFileName+'.msh',1,1)
elementsS2,IdnodeS2=silex_lib_gmsh.ReadGmshElements(MeshFileName+'.msh',1,2)
elementsS3,IdnodeS3=silex_lib_gmsh.ReadGmshElements(MeshFileName+'.msh',1,3)

# write the surface mesh in a gmsh-format file to verify if its correct
#silex_lib_gmsh.WriteResults(ResultsFileName+'_Surface_mesh',nodes,elements,2)
#silex_lib_gmsh.WriteResults(ResultsFileName+'_S1_mesh',nodes,elementsS1,1)
#silex_lib_gmsh.WriteResults(ResultsFileName+'_S2_mesh',nodes,elementsS2,1)
#silex_lib_gmsh.WriteResults(ResultsFileName+'_S3_mesh',nodes,elementsS3,1)

# Define material
Young  = 200000.0
nu     = 0.3
thickness = 1.0

# Boundary conditions

# define fixed dof
IdNodesFixed_x=IdnodeS1
IdNodesFixed_y=IdnodeS2

F=silex_lib_elt.forceonline(nodes,elementsS3,[0.0,10.0,0.0,10.0],[0.0,200.0,100.0,200.0])

toc = time.perf_counter()
print("Time for the reading data part:",toc-tic)

tic0 = time.perf_counter()
#############################################################################
#      EXPERT PART
#############################################################################
#      initialisations
#############################################################################
# get number of nodes, dof and elements from the mesh
nnodes = nodes.shape[0]
ndof   = nnodes*ndim
nelem  = elements.shape[0]
print("Number of nodes:",nnodes)
print("Number of elements:",nelem)

# define fixed dof
Fixed_Dofs = numpy.hstack([(IdNodesFixed_x-1)*2,(IdNodesFixed_y-1)*2+1])

# define free dof
SolvedDofs = numpy.setdiff1d(range(ndof),Fixed_Dofs)

# initialize displacement vector
Q=numpy.zeros(ndof)

#############################################################################
#      compute stiffness matrix
#############################################################################
tic = time.perf_counter()

Ik,Jk,Vk=silex_lib_elt.stiffnessmatrix(nodes,elements,[Young,nu,thickness])

K=scipy.sparse.csc_matrix( (Vk,(Ik,Jk)), shape=(ndof,ndof) )

toc = time.perf_counter()
print("Time to compute the stiffness matrix:",toc-tic)

#############################################################################
#       Solve the problem
#############################################################################

tic = time.perf_counter()
Q[SolvedDofs] = scipy.sparse.linalg.spsolve(K[SolvedDofs,:][:,SolvedDofs],F[SolvedDofs])
# install mumps to improve the computational time
#Q[SolvedDofs] = mumps.spsolve(K[SolvedDofs,:][:,SolvedDofs],F[SolvedDofs])
toc = time.perf_counter()
print("Time to solve the problem:",toc-tic)

#############################################################################
#       compute stress, smooth stress, strain and error
#############################################################################
tic = time.perf_counter()

SigmaElem,SigmaNodes,EpsilonElem,EpsilonNodes,ErrorElem,ErrorGlobal=silex_lib_elt.compute_stress_strain_error(nodes,elements,[Young,nu,thickness],Q)

toc = time.perf_counter()
print("Time to compute stress and error:",toc-tic)
print("The global error is:",ErrorGlobal)


#############################################################################
#         Write results to gmsh format
#############################################################################
tic = time.perf_counter()

# displacement written on 2 columns:
disp=numpy.zeros((nnodes,2))
disp[range(nnodes),0]=Q[list(range(0,ndof,2))]
disp[range(nnodes),1]=Q[list(range(1,ndof,2))]

load=numpy.zeros((nnodes,ndim))
load[range(nnodes),0]=F[list(range(0,ndof,2))]
load[range(nnodes),1]=F[list(range(1,ndof,2))]

if flag_write_fields==0:
    fields_to_write=[ [disp,'nodal',ndim,'displacement'],
                      [SigmaElem[range(nelem),[3]],'elemental',1,'Sigma V.M.'],
                      [SigmaNodes[range(nnodes),[3]],'nodal',1,'Sigma V.M. Smooth'],
                      [ErrorElem,'elemental',1,'error'],
                      ]

if flag_write_fields==1:
    fields_to_write=[ [disp,'nodal',ndim,'Displacement'],
                  [load,'nodal',2,'Force'],
                  [SigmaElem[range(nelem),[0]],'elemental',1,'Sigma xx'],
                  [SigmaElem[range(nelem),[1]],'elemental',1,'Sigma yy'],
                  [SigmaElem[range(nelem),[2]],'elemental',1,'Sigma xy'],
                  [SigmaElem[range(nelem),[3]],'elemental',1,'Sigma V.M.'],
                  [SigmaNodes[range(nnodes),[0]],'nodal',1,'Sigma xx Smooth'],
                  [SigmaNodes[range(nnodes),[1]],'nodal',1,'Sigma yy Smooth'],
                  [SigmaNodes[range(nnodes),[2]],'nodal',1,'Sigma xy Smooth'],
                  [SigmaNodes[range(nnodes),[3]],'nodal',1,'Sigma V.M. Smooth'],
                  [EpsilonElem[range(nelem),[0]],'elemental',1,'Epsilon xx'],
                  [EpsilonElem[range(nelem),[1]],'elemental',1,'Epsilon yy'],
                  [EpsilonElem[range(nelem),[2]]/2.0,'elemental',1,'Epsilon xy'],
                  [EpsilonNodes[range(nnodes),[0]],'nodal',1,'Epsilon xx Smooth'],
                  [EpsilonNodes[range(nnodes),[1]],'nodal',1,'Epsilon yy Smooth'],
                  [EpsilonNodes[range(nnodes),[2]]/2.0,'nodal',1,'Epsilon xy Smooth'],
                  [ErrorElem,'elemental',1,'Error']
                  ]

# write the mesh and the results in a gmsh-format file
silex_lib_gmsh.WriteResults(ResultsFileName,nodes,elements,eltype,fields_to_write)


toc = time.perf_counter()
print("Time to write results:",toc-tic)
print("----- END -----")



from SILEXlight import silex_lib_gmsh as sx_gmsh

def test_WriteResult():
    # sx_gmsh.WriteResults(filename=,nodes=,elttype= ,fields=)
    assert True
    
def test_ReadGmshNodes():
    # sx_gmsh.ReadGmshNodes(file=,nbcoord=)
    assert True
    
def test_ReadGmshElements():
    # sx_gmsh.ReadGmshElements(file=,elmttype=,prop=)
    assert True


from SILEXlight import silex_lib_tri3_python as sx
import numpy as np

def test_det33_ligne_de_un():
    mat = np.random.randn(3,3)
    mat[0,:] = 1
    np.testing.assert_allclose(sx.det33_ligne_de_un(mat[1:3,:]), np.linalg.det(mat))
    
def test_stiffnessmatrix():
    assert True
    
def test_compute_stress_strain_error():
    assert True
    
def test_forceonline():
    assert True
    



def WriteResults(filename, nodes, elements, elttype, fields=[]):
    """Write mesh and results in a gmsh-format file.

    syntax:
        WriteResults(filename,nodes,elements,elttype,fields)

    inputs:
        filename : string, name of the gmsh file (with no extension),
                            may contain a repertory name
        nodes    : nodes coordinates
        elements : connectivity
        elttype  : element type define as in gmsh (refer to gmsh documentation for numbering)
                    1 : 2-node line
                    2 : 3-node triangle
                    3 : 4-node quadrangle
                    4 : 4-node tetrahedron.
                    5 : 8-node hexahedron.
                    6 : 6-node prism.
                    7 : 5-node pyramid.
                    8 : 3-node second order line (2 nodes associated with the vertices and 1 with the edge).
                    9 : 6-node second order triangle (3 nodes associated with the vertices and 3 with the edges).
                    10: 9-node second order quadrangle (4 nodes associated with the vertices, 4 with the edges and 1 with the face).
                    11 : 10-node second order tetrahedron (4 nodes associated with the vertices and 6 with the edges).
                    12 : 27-node second order hexahedron (8 nodes associated with the vertices, 12 with the edges, 6 with the faces and 1 with the volume).
                    13 : 18-node second order prism (6 nodes associated with the vertices, 9 with the edges and 3 with the quadrangular faces).
                    14 : 14-node second order pyramid (5 nodes associated with the vertices, 8 with the edges and 1 with the quadrangular face).
                    15 : 1-node point.
                    16 : 8-node second order quadrangle (4 nodes associated with the vertices and 4 with the edges).
                    17 : 20-node second order hexahedron (8 nodes associated with the vertices and 12 with the edges).
                    18 : 15-node second order prism (6 nodes associated with the vertices and 9 with the edges).
                    19 : 13-node second order pyramid (5 nodes associated with the vertices and 8 with the edges).

        fields (optional)  : list of the fields to write, syntax:
            fields=[[variable_name1,'nodal' or'elemental' ,number of values per node,'name 1'],
                    [variable_name2,'nodal' or'elemental' ,number of values per node,'name 2'],
                    ...
                    ]

    Returns string."""

    f = open(filename+'.msh', 'w')

    nnodes = nodes.shape[0]
    ndof = nnodes*2
    nelem = elements.shape[0]

    f.write('$MeshFormat\n')
    f.write('2.2 0 8\n')
    f.write('$EndMeshFormat\n')
    f.write('$Nodes\n')
    f.write('%d\n' % (nnodes))

    ###############
    # write nodes #
    ###############
    # (2d)
    if nodes.shape[1] == 2:
        for i in range(nnodes):
            f.write('%d %s %s 0.0\n' % (i+1,
                                        nodes[i, 0],
                                        nodes[i, 1]))

    # (3d)
    if nodes.shape[1] == 3:
        for i in range(nnodes):
            f.write('%d %s %s %s\n' % (i+1,
                                       nodes[i, 0],
                                       nodes[i, 1],
                                       nodes[i, 2]))

    f.write('$EndNodes\n')

    ##################
    # write elements #
    ##################
    f.write('$Elements\n')
    f.write('%d\n' % (nelem))
    # 2-node truss elements
    if elttype == 1:
        for e in range(nelem):
            f.write('%d 1 3 1 1 0 %d %d\n' % (e+1,
                                              elements[e, 0],
                                              elements[e, 1]))

    # 3-node triangle
    if elttype == 2:
        for e in range(nelem):
            f.write('%d 2 3 1 1 0 %d %d %d\n' % (e+1,
                                                 elements[e, 0],
                                                 elements[e, 1],
                                                 elements[e, 2]))

    # 4-node quadrangle
    if elttype == 3:
        for e in range(nelem):
            f.write('%d 3 3 1 1 0 %d %d %d %d\n' % (e+1,
                                                    elements[e, 0],
                                                    elements[e, 1],
                                                    elements[e, 2],
                                                    elements[e, 3]))
    # 4-node tetrahedron
    if elttype == 4:
        for e in range(nelem):
            f.write('%d 4 3 1 1 0 %d %d %d %d\n' % (e+1,
                                                    elements[e, 0],
                                                    elements[e, 1],
                                                    elements[e, 2],
                                                    elements[e, 3]))

    # 8-node hexahedron
    if elttype == 5:
        for e in range(nelem):
            f.write('%d 5 3 1 1 0 %d %d %d %d %d %d %d %d\n' % (e+1,
                                                                elements[e, 0],
                                                                elements[e, 1],
                                                                elements[e, 2],
                                                                elements[e, 3],
                                                                elements[e, 4],
                                                                elements[e, 5],
                                                                elements[e, 6],
                                                                elements[e, 7]))
    # 3-node line
    if elttype == 8:
        for e in range(nelem):
            f.write('%d 8 3 1 1 0 %d %d %d \n' % (e+1,
                                                  elements[e, 0],
                                                  elements[e, 1],
                                                  elements[e, 2]))
    # 6-node triangle
    if elttype == 9:
        for e in range(nelem):
            f.write('%d 9 3 1 1 0 %d %d %d %d %d %d\n' % (e+1,
                                                          elements[e, 0],
                                                          elements[e, 1],
                                                          elements[e, 2],
                                                          elements[e, 3],
                                                          elements[e, 4],
                                                          elements[e, 5]))

    # 10-node tetrahedron
    if elttype == 11:
        for e in range(nelem):
            f.write('%d 11 3 1 1 0 %d %d %d %d %d %d %d %d %d %d\n' % (e+1,
                                                                       elements[e, 0],
                                                                       elements[e, 1],
                                                                       elements[e, 2],
                                                                       elements[e, 3],
                                                                       elements[e, 4],
                                                                       elements[e, 5],
                                                                       elements[e, 6],
                                                                       elements[e, 7],
                                                                       elements[e, 8],
                                                                       elements[e, 9]))

    f.write('$EndElements\n')

    #################
    # write results #
    #################
    for p in range(len(fields)):
        values = fields[p][0]
        valuestype = fields[p][1]
        nbpernode = fields[p][2]
        name = fields[p][3]
        # f.write('View[%d].Visible = 0;\n' % p)
        if valuestype == 'nodal':
            f.write('$NodeData\n')
            f.write('1\n')
            f.write('"%s"\n' % name)
            f.write('1\n')
            f.write('0.0\n')
            f.write('3\n')
            f.write('0\n')
            if nbpernode == 1:
                f.write('1\n')
            if nbpernode == 2:
                f.write('3\n')
            if nbpernode == 3:
                f.write('3\n')
            f.write('%d\n' % (nnodes))
            if nbpernode == 1:
                for i in range(nnodes):
                    f.write('%d %s\n' % (i+1, values[i]))

            if nbpernode == 2:
                for i in range(nnodes):
                    f.write('%d %s %s 0.0\n' %
                            (i+1, values[i, 0], values[i, 1]))

            if nbpernode == 3:
                for i in range(nnodes):
                    f.write('%d %s %s %s\n' %
                            (i+1, values[i, 0], values[i, 1], values[i, 2]))

            f.write('$EndNodeData\n')

        if valuestype == 'elemental':
            f.write('$ElementData\n')
            f.write('1\n')
            f.write('"%s"\n' % name)
            f.write('1\n')
            f.write('0.0\n')
            f.write('3\n')
            f.write('0\n')
            if nbpernode == 1:
                f.write('1\n')
            if nbpernode == 2:
                f.write('3\n')
            if nbpernode == 3:
                f.write('3\n')
            f.write('%d\n' % (nelem))

            if nbpernode == 1:
                for e in range(nelem):
                    f.write('%d %s\n' % (e+1, values[e]))

            if nbpernode == 2:
                for e in range(nelem):
                    f.write('%d %s %s\n' % (e+1, values[e, 0], values[e, 1]))

            if nbpernode == 3:
                for e in range(nelem):
                    f.write('%d %s %s %s\n' %
                            (e+1, values[e, 0], values[e, 1], values[e, 2]))

            f.write('$EndElementData\n')

    f.close()
    return

##############################################################
# FUNCTION : read nodes from GMSH output file
##############################################################


def ReadGmshNodes(file, nbcoord):
    import string
    import numpy
    f = open(file, 'r')
    i = 0
    j = 0
    # nodes=[]
    goodline = 0
    nbnodes = 0
    for l in f:
        # s=string.split(l)
        s = l.split()
        if (goodline == 1):
            if (nbnodes == 0):
                nbnodes = int(s[0])
                nodes = numpy.zeros((nbnodes, nbcoord))
            else:
                if (i < nbnodes):
                    if nbcoord == 3:
                        nodes[i, 0] = float(s[1])
                        nodes[i, 1] = float(s[2])
                        nodes[i, 2] = float(s[3])
                        i = i+1

                    if nbcoord == 2:
                        nodes[i, 0] = float(s[1])
                        nodes[i, 1] = float(s[2])
                        i = i+1

        if (s[0] == '$Nodes'):
            goodline = 1

    f.close()
    return nodes

##############################################################
# FUNCTION : read elements from GMSH output file
##############################################################


def ReadGmshElements(file, elmttype, prop):
    import string
    import numpy
    f = open(file, 'r')
    i = -1
    j = 0
    elements = []
    goodline = 0
    if (elmttype == 11):
        nbnodes = 10
    if (elmttype == 9):
        nbnodes = 6
    if (elmttype == 8):
        nbnodes = 3
    if (elmttype == 5):
        nbnodes = 8
    if (elmttype == 4):
        nbnodes = 4
    if (elmttype == 2):
        nbnodes = 3
    if (elmttype == 3):
        nbnodes = 4
    if (elmttype == 1):
        nbnodes = 2

    for l in f:
        # s=string.split(l)
        s = l.split()
        if (goodline == 1):
            if (i == -1):
                nbelem = int(s[0])
                i = 0
                elements = numpy.zeros((nbelem, nbnodes), dtype=int)
            else:
                if (i < nbelem):
                    if (int(s[3]) == prop):
                        if (int(s[1]) == elmttype):
                            for k in range(nbnodes):
                                elements[j, k] = int(s[5+k])
                            j = j+1
                    i = i+1

        if (s[0] == '$Elements'):
            goodline = 1

    f.close()
    return elements[list(range(j)), :][:, list(range(nbnodes))], numpy.unique(elements[list(range(j)), :][:, list(range(nbnodes))])




import numpy as np


def det33_ligne_de_un(a):
    return a[0, 1]*a[1, 2]-a[0, 2]*a[1, 1]-a[0, 0]*a[1, 2]+a[0, 0]*a[1, 1]+a[1, 0]*a[0, 2]-a[1, 0]*a[0, 1]


def stiffnessmatrix(nodes, elements, material):
    nbnodes = nodes.shape[0]
    nbelem = elements.shape[0]

    Young = material[0]
    nu = material[1]
    thickness = material[2]

    Ik = np.zeros(6*6*nbelem, dtype=int)
    Jk = np.zeros(6*6*nbelem, dtype=int)
    Vk = np.zeros(6*6*nbelem, dtype=float)
    a23 = np.zeros((2, 3), dtype=float)
    b = np.zeros(3, dtype=float)
    c = np.zeros(3, dtype=float)
    BB = np.zeros((3, 6), dtype=float)
    dofelem = np.zeros(6, dtype=int)
    dofx = np.zeros(3, dtype=int)
    dofy = np.zeros(3, dtype=int)
    idnodes = np.zeros(3, dtype=int)

    CC = np.array([[Young/(1-nu**2), nu*Young/(1-nu**2), 0],
                      [nu*Young/(1-nu**2), Young/(1-nu**2), 0],
                      [0, 0, Young/(2*(1+nu))]
                      ])

    p = 0
    for e in range(nbelem):
        idnodes[:] = elements[e, :]-1
        dofx[:] = (idnodes)*2
        dofy[:] = (idnodes)*2+1

        dofelem[0] = dofx[0]
        dofelem[1] = dofx[1]
        dofelem[2] = dofx[2]
        dofelem[3] = dofy[0]
        dofelem[4] = dofy[1]
        dofelem[5] = dofy[2]

        X = nodes[idnodes, 0]
        Y = nodes[idnodes, 1]

        a23[0, 0] = X[0]
        a23[0, 1] = X[1]
        a23[0, 2] = X[2]
        a23[1, 0] = Y[0]
        a23[1, 1] = Y[1]
        a23[1, 2] = Y[2]
        det_of_sys = det33_ligne_de_un(a23)
        Area = abs(0.5*det_of_sys)

        b[0] = -(Y[2]-Y[1])/det_of_sys
        c[0] = +(X[2]-X[1])/det_of_sys

        b[1] = +(Y[2]-Y[0])/det_of_sys
        c[1] = -(X[2]-X[0])/det_of_sys

        b[2] = -(Y[1]-Y[0])/det_of_sys
        c[2] = +(X[1]-X[0])/det_of_sys

        BB[0, 0] = b[0]
        BB[0, 1] = b[1]
        BB[0, 2] = b[2]
        BB[0, 3] = 0.0
        BB[0, 4] = 0.0
        BB[0, 5] = 0.0
        BB[1, 0] = 0.0
        BB[1, 1] = 0.0
        BB[1, 2] = 0.0
        BB[1, 3] = c[0]
        BB[1, 4] = c[1]
        BB[1, 5] = c[2]
        BB[2, 0] = c[0]
        BB[2, 1] = c[1]
        BB[2, 2] = c[2]
        BB[2, 3] = b[0]
        BB[2, 4] = b[1]
        BB[2, 5] = b[2]

        ke = np.dot(BB.T, np.dot(CC, BB))*Area*thickness

        for i in range(6):
            #for j in range(12):
            Ik[p:(p+6)] = dofelem[i]
            Jk[p:(p+6)] = dofelem[:]
            Vk[p:(p+6)] = ke[i, :]
            p = p+6
            #Ik.append(dofelem[i])
            #Jk.append(dofelem[j])
            #Vk.append(ke[i,j])

    return Ik, Jk, Vk

############################################################################


def compute_stress_strain_error(nodes, elements, material, QQ):
    nbnodes = nodes.shape[0]
    nbelem = elements.shape[0]

    Young = material[0]
    nu = material[1]
    thickness = material[2]

    CC = np.array([[Young/(1-nu**2), nu*Young/(1-nu**2), 0],
                      [nu*Young/(1-nu**2), Young/(1-nu**2), 0],
                      [0, 0, Young/(2*(1+nu))]
                      ])
    CCinv = np.array([[1.0/Young, -nu/Young, 0.0],
                         [-nu/Young, 1.0/Young, 0.0],
                         [0.0, 0.0, (2*(1+nu))/Young]
                         ])

    rg = np.zeros(3, dtype=float)
    sg = np.zeros(3, dtype=float)
    tg = np.zeros(3, dtype=float)
    wg = np.zeros(3, dtype=float)

    nodalweight = np.zeros(nbnodes, dtype=float)
    sig_smooth = np.zeros((nbnodes, 4), dtype=float)
    a23 = np.zeros((2, 3), dtype=float)
    b = np.zeros(3, dtype=float)
    c = np.zeros(3, dtype=float)
    BB = np.zeros((3, 6), dtype=float)
    dofelem = np.zeros(6, dtype=int)
    dofx = np.zeros(3, dtype=int)
    dofy = np.zeros(3, dtype=int)
    idnodes = np.zeros(3, dtype=int)

    Sigma = np.zeros((nbelem, 4), dtype=float)
    Epsilon = np.zeros((nbelem, 3), dtype=float)
    EpsilonNodes = np.zeros((nbnodes, 3), dtype=float)
    ErrElem = np.zeros(nbelem, dtype=float)
    NormSigElt = np.zeros(nbelem, dtype=float)
    ErrGlob = 0.0
    NormSig = 0.0

    NN = np.zeros(3, dtype=float)
    sig_smooth_Gauss = np.zeros(3, dtype=float)
    sigdiff = np.zeros(3, dtype=float)

    # Quadrature points
    rg[0] = 2.0/3.0
    sg[0] = 1.0/6.0
    tg[0] = 1.0/6.0
    wg[0] = 0.5*1.0/3.0

    rg[1] = 1.0/6.0
    sg[1] = 2.0/3.0
    tg[1] = 1.0/6.0
    wg[1] = 0.5*1.0/3.0

    rg[2] = 1.0/6.0
    sg[2] = 1.0/6.0
    tg[2] = 2.0/3.0
    wg[2] = 0.5*1.0/3.0

    for e in range(nbelem):
        idnodes[:] = elements[e, :]-1
        dofx[:] = (idnodes)*2
        dofy[:] = (idnodes)*2+1

        dofelem[0] = dofx[0]
        dofelem[1] = dofx[1]
        dofelem[2] = dofx[2]
        dofelem[3] = dofy[0]
        dofelem[4] = dofy[1]
        dofelem[5] = dofy[2]

        X = nodes[idnodes, 0]
        Y = nodes[idnodes, 1]
        Uelem = QQ[dofelem]

        a23[0, 0] = X[0]
        a23[0, 1] = X[1]
        a23[0, 2] = X[2]
        a23[1, 0] = Y[0]
        a23[1, 1] = Y[1]
        a23[1, 2] = Y[2]
        det_of_sys = det33_ligne_de_un(a23)
        Area = abs(0.5*det_of_sys)

        b[0] = -(Y[2]-Y[1])/det_of_sys
        c[0] = +(X[2]-X[1])/det_of_sys

        b[1] = +(Y[2]-Y[0])/det_of_sys
        c[1] = -(X[2]-X[0])/det_of_sys

        b[2] = -(Y[1]-Y[0])/det_of_sys
        c[2] = +(X[1]-X[0])/det_of_sys

        BB[0, 0] = b[0]
        BB[0, 1] = b[1]
        BB[0, 2] = b[2]
        BB[0, 3] = 0.0
        BB[0, 4] = 0.0
        BB[0, 5] = 0.0
        BB[1, 0] = 0.0
        BB[1, 1] = 0.0
        BB[1, 2] = 0.0
        BB[1, 3] = c[0]
        BB[1, 4] = c[1]
        BB[1, 5] = c[2]
        BB[2, 0] = c[0]
        BB[2, 1] = c[1]
        BB[2, 2] = c[2]
        BB[2, 3] = b[0]
        BB[2, 4] = b[1]
        BB[2, 5] = b[2]

        EpsilonElem = np.dot(BB, Uelem)
        SigmaElem = np.dot(CC, EpsilonElem)

        Epsilon[e, 0] = EpsilonElem[0]
        Epsilon[e, 1] = EpsilonElem[1]
        Epsilon[e, 2] = EpsilonElem[2]
        Sigma[e, 0] = SigmaElem[0]
        Sigma[e, 1] = SigmaElem[1]
        Sigma[e, 2] = SigmaElem[2]
        # Von Mises
        Sigma[e, 3] = np.sqrt(1.5*(SigmaElem[0]**2+SigmaElem[1] **
                                 2+2.0*SigmaElem[2]**2)-0.5*(SigmaElem[0]+SigmaElem[1])**2)

        for i in range(3):
            nodalweight[idnodes[i]] = nodalweight[idnodes[i]]+Area
            sig_smooth[idnodes[i], :] = sig_smooth[idnodes[i], :] + \
                Sigma[e, :]*Area

    # norm the smooth stress
    for i in range(nbnodes):
        for j in range(4):
            sig_smooth[i, j] = sig_smooth[i, j]/nodalweight[i]

        for j in range(3):
            EpsilonNodes[i, j] = CCinv[j, 0]*sig_smooth[i, 0] + \
                CCinv[j, 1]*sig_smooth[i, 1]+CCinv[j, 2]*sig_smooth[i, 2]

    # compute error
    for e in range(nbelem):
        idnodes[:] = elements[e, :]-1

        X = nodes[idnodes, 0]
        Y = nodes[idnodes, 1]

        a23[0, 0] = X[0]
        a23[0, 1] = X[1]
        a23[0, 2] = X[2]
        a23[1, 0] = Y[0]
        a23[1, 1] = Y[1]
        a23[1, 2] = Y[2]
        det_of_sys = det33_ligne_de_un(a23)
        Area = abs(0.5*det_of_sys)

        for g in range(3):
            for j in range(3):
                sig_smooth_Gauss[j] = sig_smooth[idnodes[0], j]*rg[g] + \
                    sig_smooth[idnodes[1], j]*sg[g] + \
                    sig_smooth[idnodes[2], j]*tg[g]

            sigdiff[:] = sig_smooth_Gauss[:]-Sigma[e, 0:3]
            ErrElem[e] = ErrElem[e] + \
                np.dot(sigdiff, np.dot(CCinv, sigdiff))*Area/0.5*wg[g]
            NormSigElt[e] = NormSigElt[e]+np.dot(
                sig_smooth_Gauss, np.dot(CCinv, sig_smooth_Gauss))*Area/0.5*wg[g]

    ErrElem = ErrElem/sum(NormSigElt)
    ErrGlob = np.sqrt(sum(ErrElem))

    return Sigma, sig_smooth, Epsilon, EpsilonNodes, ErrElem, ErrGlob

############################################################################


def forceonline(nodes, elements, fs, pts):
    # nodes: node coordinates
    # elements: 2-node line elements on which the force is applied
    # fs=[ surf. load on pt1 x-direc  , surf. load on pt1 y-direc , surf. load on pt2 x-direc  , surf. load on pt2 y-direc ]
    # fs : units = Newton per length
    # pts=[ pt 1 x , pt 1 y ,pt 2 x ,pt 2 y]
    nelem = elements.shape[0]
    rg = [-1.0/np.sqrt(3.0), 1.0/np.sqrt(3.0)]
    wg = [1.0, 1.0]

    F = np.zeros(nodes.shape[0]*2)

    for e in range(nelem):

        idnodes = elements[e, :]

        dofx = (idnodes-1)*2
        dofy = (idnodes-1)*2+1

        xnodes = nodes[idnodes-1, 0]
        ynodes = nodes[idnodes-1, 1]
        lelem = np.sqrt((xnodes[1]-xnodes[0])**2+(ynodes[1]-ynodes[0])**2)

        forceelem = np.zeros((4, 1), dtype=float)

        for g in range(2):
            N = [(1-rg[g])*0.5, (1+rg[g])*0.5]
            x = N[0]*xnodes[0]+N[1]*xnodes[1]
            y = N[0]*ynodes[0]+N[1]*ynodes[1]

            l0 = np.sqrt((pts[0]-x)**2+(pts[1]-y)**2)
            l1 = np.sqrt((pts[2]-x)**2+(pts[3]-y)**2)

            forcex = (l1*fs[0]+l0*fs[2])/(l0+l1)
            forcey = (l1*fs[1]+l0*fs[3])/(l0+l1)

            Phi = np.array([[N[0], 0.0, N[1], 0.0],
                               [0.0, N[0], 0.0, N[1]]
                               ])
            forcegausspt = np.array([[forcex], [forcey]])

            forceelem = forceelem + \
                np.dot(Phi.T, forcegausspt)*wg[g]*0.5*lelem

        F[dofx[0]] = F[dofx[0]]+forceelem[0]
        F[dofy[0]] = F[dofy[0]]+forceelem[1]
        F[dofx[1]] = F[dofx[1]]+forceelem[2]
        F[dofy[1]] = F[dofy[1]]+forceelem[3]

    return F



#############################################################################
#      Import libraries
#############################################################################
import string
import time
import numpy
import scipy.sparse
import scipy.sparse.linalg


# Choose between the Fortran or the Python librairie:

# import silex_lib_tet4_fortran as silex_lib_elt
from SILEXlight import silex_lib_tet4_python as silex_lib_elt
from SILEXlight import silex_lib_gmsh

# install mumps to solve the system more quickly
# import mumps

#############################################################################
print("SILEX CODE - computation of a with tet4 elements")
#############################################################################

tic = time.perf_counter()
#############################################################################
#      USER PART: Import mesh, boundary conditions and material
#############################################################################

# Input mesh: define the name of the mesh file (*.msh)
MeshFileName = 'piston-tet4'

# Output result file: define the name of the result file (*.msh)
ResultsFileName = 'Results_piston_tet4'

# choose the element type
eltype = 4

# choose geometry dimension
ndim = 3

# choose the results in the results file
flag_write_fields = 0

# read the mesh from gmsh
nodes = silex_lib_gmsh.ReadGmshNodes(MeshFileName+'.msh', ndim)
elements, Idnodes = silex_lib_gmsh.ReadGmshElements(
    MeshFileName+'.msh', eltype, 5)

# read surfaces where to impose boundary conditions
elementsS1, IdnodeS1 = silex_lib_gmsh.ReadGmshElements(
    MeshFileName+'.msh', 2, 1)
elementsS2, IdnodeS2 = silex_lib_gmsh.ReadGmshElements(
    MeshFileName+'.msh', 2, 2)
elementsS3, IdnodeS3 = silex_lib_gmsh.ReadGmshElements(
    MeshFileName+'.msh', 2, 3)
elementsS4, IdnodeS4 = silex_lib_gmsh.ReadGmshElements(
    MeshFileName+'.msh', 2, 4)

# write the surface mesh in a gmsh-format file to verify if its correct
# silex_lib_gmsh.WriteResults(ResultsFileName+'Volum',nodes,elements,4)
# silex_lib_gmsh.WriteResults(ResultsFileName+'surf1',nodes,elementsS1,2)
# silex_lib_gmsh.WriteResults(ResultsFileName+'surf2',nodes,elementsS2,2)
# silex_lib_gmsh.WriteResults(ResultsFileName+'surf3',nodes,elementsS3,2)
# silex_lib_gmsh.WriteResults(ResultsFileName+'surf4',nodes,elementsS4,2)

# Define material
Young = 200000.0
nu = 0.3

# Boundary conditions
IdNodesFixed_x = IdnodeS3
IdNodesFixed_y = IdnodeS1
IdNodesFixed_z = IdnodeS2

# compute external forces from pressure
press = -10  # MPa
# give the direction of the surfacic load:
#          if [0.0,0.0,0.0] then the local normal to the surface is used
#          otherwise, the direction is normalized to 1
direction = [0.0, 0.0, 0.0]
F = silex_lib_elt.forceonsurface(nodes, elementsS4, press, direction)


toc = time.perf_counter()
print("Time for the user part:", toc-tic)

#############################################################################
#      EXPERT PART
#############################################################################
#      initialisations
#############################################################################
# get number of nodes, dof and elements from the mesh
nnodes = nodes.shape[0]
ndof = nnodes*ndim
nelem = elements.shape[0]
print("Number of nodes:", nnodes)
print("Number of elements:", nelem)

# define fixed dof
Fixed_Dofs = numpy.hstack(
    [(IdNodesFixed_x-1)*3, (IdNodesFixed_y-1)*3+1, (IdNodesFixed_z-1)*3+2])

# define free dof
SolvedDofs = numpy.setdiff1d(range(ndof), Fixed_Dofs)

# initialize displacement vector
Q = numpy.zeros(ndof)

#############################################################################
#      compute stiffness matrix
#############################################################################
tic0 = time.perf_counter()
tic = time.perf_counter()
Ik, Jk, Vk = silex_lib_elt.stiffnessmatrix(nodes, elements, [Young, nu])
toc = time.perf_counter()

K = scipy.sparse.csr_matrix((Vk, (Ik, Jk)), shape=(ndof, ndof), dtype=float)
print("Time to compute the stiffness matrix:", toc-tic)

#############################################################################
#       Solve the problem
#############################################################################

tic = time.perf_counter()
Q[SolvedDofs] = scipy.sparse.linalg.spsolve(
    K[SolvedDofs, :][:, SolvedDofs], F[SolvedDofs])
# install mumps to improve the computational time
# import mumps
# Q[SolvedDofs] = mumps.spsolve(K[SolvedDofs,:][:,SolvedDofs],F[SolvedDofs])
toc = time.perf_counter()
print("Time to solve the problem:", toc-tic)

#############################################################################
#       compute smooth stress and error in elements
#############################################################################
tic = time.perf_counter()

SigmaElem, SigmaNodes, EpsilonElem, EpsilonNodes, ErrorElem, ErrorGlobal = silex_lib_elt.compute_stress_strain_error(
    nodes, elements, [Young, nu], Q)

toc = time.perf_counter()
print("Time to compute stress and error:", toc-tic)
print("The global error is:", ErrorGlobal)
print("Total time for the computational part:", toc-tic0)

#############################################################################
#         Write results to gmsh format
#############################################################################
tic = time.perf_counter()

# displacement written on 3 columns:
disp = numpy.zeros((nnodes, ndim))
disp[range(nnodes), 0] = Q[list(range(0, ndof, 3))]
disp[range(nnodes), 1] = Q[list(range(1, ndof, 3))]
disp[range(nnodes), 2] = Q[list(range(2, ndof, 3))]

# external load written on 3 columns:
load = numpy.zeros((nnodes, ndim))
load[range(nnodes), 0] = F[list(range(0, ndof, 3))]
load[range(nnodes), 1] = F[list(range(1, ndof, 3))]
load[range(nnodes), 2] = F[list(range(2, ndof, 3))]

if flag_write_fields == 0:
    fields_to_write = [[disp, 'nodal', ndim, 'displacement'],
                       [SigmaElem[range(nelem), [6]],
                        'elemental', 1, 'Sigma V.M.'],
                       [SigmaNodes[range(nnodes), [6]], 'nodal',
                        1, 'Smooth Sigma V.M.'],
                       [ErrorElem, 'elemental', 1, 'error'],
                       ]

if flag_write_fields == 1:
    fields_to_write = [[disp, 'nodal', ndim, 'displacement'],
                       [load, 'nodal', ndim, 'Force'],
                       [SigmaElem[range(nelem), [0]],
                        'elemental', 1, 'Sigma 11'],
                       [SigmaElem[range(nelem), [1]],
                        'elemental', 1, 'Sigma 22'],
                       [SigmaElem[range(nelem), [2]],
                        'elemental', 1, 'Sigma 33'],
                       [SigmaElem[range(nelem), [3]],
                        'elemental', 1, 'Sigma 23'],
                       [SigmaElem[range(nelem), [4]],
                        'elemental', 1, 'Sigma 13'],
                       [SigmaElem[range(nelem), [5]],
                        'elemental', 1, 'Sigma 12'],
                       [SigmaElem[range(nelem), [6]],
                        'elemental', 1, 'Sigma V.M.'],
                       [SigmaNodes[range(nnodes), [0]], 'nodal',
                        1, 'Smooth Sigma 11'],
                       [SigmaNodes[range(nnodes), [1]], 'nodal',
                        1, 'Smooth Sigma 22'],
                       [SigmaNodes[range(nnodes), [2]], 'nodal',
                        1, 'Smooth Sigma 33'],
                       [SigmaNodes[range(nnodes), [3]], 'nodal',
                        1, 'Smooth Sigma 23'],
                       [SigmaNodes[range(nnodes), [4]], 'nodal',
                        1, 'Smooth Sigma 13'],
                       [SigmaNodes[range(nnodes), [5]], 'nodal',
                        1, 'Smooth Sigma 12'],
                       [SigmaNodes[range(nnodes), [6]], 'nodal',
                        1, 'Smooth Sigma V.M.'],
                       [EpsilonElem[range(nelem), [0]],
                        'elemental', 1, 'Epsilon 11'],
                       [EpsilonElem[range(nelem), [1]],
                        'elemental', 1, 'Epsilon 22'],
                       [EpsilonElem[range(nelem), [2]],
                        'elemental', 1, 'Epsilon 33'],
                       [EpsilonElem[range(nelem), [3]]/2.0,
                        'elemental', 1, 'Epsilon 23'],
                       [EpsilonElem[range(nelem), [4]]/2.0,
                        'elemental', 1, 'Epsilon 13'],
                       [EpsilonElem[range(nelem), [5]]/2.0,
                        'elemental', 1, 'Epsilon 12'],
                       [EpsilonNodes[range(nnodes), [0]],
                        'nodal', 1, 'Smooth Epsilon 11'],
                       [EpsilonNodes[range(nnodes), [1]],
                        'nodal', 1, 'Smooth Epsilon 22'],
                       [EpsilonNodes[range(nnodes), [2]],
                        'nodal', 1, 'Smooth Epsilon 33'],
                       [EpsilonNodes[range(nnodes), [3]]/2.0, 'nodal',
                        1, 'Smooth Epsilon 23'],
                       [EpsilonNodes[range(nnodes), [4]]/2.0, 'nodal',
                        1, 'Smooth Epsilon 13'],
                       [EpsilonNodes[range(nnodes), [5]]/2.0, 'nodal',
                        1, 'Smooth Epsilon 12'],
                       [ErrorElem, 'elemental', 1, 'error'],
                       ]

# write the mesh and the results in a gmsh-format file
silex_lib_gmsh.WriteResults(ResultsFileName, nodes,
                            elements, eltype, fields_to_write)

toc = time.perf_counter()
print("Time to write results:", toc-tic)
print("----- END -----")



from SILEXlight import silex_lib_tri3_fortran as sx_tri3
import numpy as np

# subroutine ElementalStiffness(X,Y,thickness,CC,ke)
# subroutine StiffnessMatrix(nbnodes,nodes,
# subroutine ComputeElementalStress(X,Y,CC,Q,Sig,Eps,area)
# subroutine Compute_stress_strain_error(nbnodes,nodes,
# subroutine forceonline(nbnodes,nodes,nbelem,elements,


def test_det33_ligne_de_un():
    mat = np.random.randn(3, 3)
    mat[0, :] = 1
    np.testing.assert_almost_equal(
        sx_tri3.det33_ligne_de_un(mat[1:3, :]), np.linalg.det(mat))


def test_ElementalStiffness():
    assert True


def test_StiffnessMatrix():
    assert True


def test_ComputeElementalStress():
    assert True


def test_Compute_stress_strain_error():
    assert True


def test_forceonline():
    assert True
